<?php

namespace Drupal\seo_keyword_links\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Config form class for seo_keyword_links module.
 *
 * @package Drupal\seo_keyword_links\Form
 */
class SeoSettingsForm extends ConfigFormBase {

  /**
   * Implements hook_install().
   */
  protected function getEditableConfigNames() {
    return ['seo_keyword_links.settings'];
  }

  /**
   * Implements \Drupal\Core\Form\FormInterface::getFormID().
   */
  public function getFormId() {
    return 'seo_settings_form';
  }

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new instance of the class.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
    $container->get('entity_type.manager')
    );
  }

  /**
   * Implements hook_form().
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('seo_keyword_links.settings');

    $form['links'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Links'),
      '#default_value' => $this->walkLinks($config->get('links')),
      '#description' => $this->t('Please enter each line with the following format "Keyword -- http://mylink.com"'),
    ];

    $contentTypes = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    $contentTypesList = array_reduce($contentTypes, function ($result, $contentType) {
      return $result + [$contentType->id() => $contentType->label()];
    }, []);

    $form['node_types'] = [
      '#type' => 'checkboxes',
      '#options' => $contentTypesList,
      '#title' => $this->t('Node types'),
      '#default_value' => $config->get("node_types"),
      '#description' => $this->t('Select the content type to apply the desired behaviour.'),
    ];

    $blank_bool = (bool) $config->get('blank');
    $form['blank'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Open link in new tab?'),
      '#size' => 10,
      '#maxlength' => 255,
      '#default_value' => $blank_bool,
      '#description' => $this->t('Check this if you want all your links opened in a new tab. 
        Uncheck if you want your links run in the same window.<br><font color="red">Format <b>Full HTML</b> 
        for body text is <i>mandatory</i> if you want this</font>'),
    ];

    $form['case'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Disregard uppercase and lowercase in keywords?'),
      '#size' => 10,
      '#maxlength' => 255,
      '#default_value' => $config->get("case"),
      '#description' => $this->t('Check this if you want to disregard uppercase and lowercase in keywords.'),
    ];

    $form['class'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Add class to links?'),
      '#size' => 10,
      '#maxlength' => 255,
      '#default_value' => $config->get("class"),
      '#description' => $this->t('Check this if you want to add a class to the generated link.<br><font color="red">Format <b>Full HTML</b> for body text is <i>mandatory</i> if you want this</font>'),
      '#suffix' => $this->t('Clear cache after changing any of these options'),
    ];

    $form['class_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Enter your desired class name for these links'),
      '#default_value' => $config->get("class_name"),
      '#states' => [
        'invisible' => [':input[name="class"]' => ['checked' => FALSE]],
      ],
    ];

    $form['cc'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Clear cache on submit form ?'),
      '#default_value' => $config->get("cc"),
      '#description' => $this->t('Check this if you want to clear cache (mandatory) now.'),
      '#suffix' => $this->t('Clear cache after changing any of these options'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    if ($form_state->getValue('cc')) {
      \Drupal::cache()->invalidateAll();
      $message = "Cache was cleared !!";

      if (function_exists('drupal_set_message')) {
        drupal_set_message($message, "status", TRUE);
      }
      else {
        \Drupal::messenger()->addMessage($message);
      }
    }

    $content_types = array_filter($form_state->getValue('node_types'), function ($content_type) {
      return $content_type !== "0";
    });

    $this->config('seo_keyword_links.settings')
      ->set('links', $this->codeLinks(explode("\n", $form_state->getValue('links'))))
      ->set('node_types', $content_types)
      ->set('blank', $form_state->getValue('blank'))
      ->set('case', $form_state->getValue('case'))
      ->set('class', $form_state->getValue('class'))
      ->set('class_name', $form_state->getValue('class_name'))
      ->set('cc', $form_state->getValue('cc'))
      ->save();
  }

  /**
   * Iterates array, returns string of concatenated key-value pairs with '--'.
   *
   * @param array $arr
   *   The array to iterate over.
   *
   * @return string
   *   A string containing the concatenated elements of the array.
   */
  private function walkLinks(array $arr): string {
    // Initialize variable for concatenating values.
    $result = '';

    // Iterate over the array and concatenate keys and values.
    foreach ($arr as $key => $value) {
      $result .= $key . ' -- ' . $value . "\n";
    }

    // Return the resulting string.
    return $result;
  }

  /**
   * Extracts links from an array of strings as an associative array.
   *
   * @param array $arr
   *   Array of strings: links in format "title -- URL.
   *
   * @return array
   *   Associative array: link titles as keys, link URLs as values.
   */
  private function codeLinks(array $arr): array {
    // Initialize the result array.
    $result = [];

    // Iterate over the array and extract the link titles and URLs.
    foreach ($arr as $value) {
      $parts = explode(' -- ', $value);
      if (isset($parts[0]) && isset($parts[1])) {
        $result[$parts[0]] = $parts[1];
      }
    }

    // Return the resulting array.
    return $result;
  }

}
